import QtQuick 2.9
import org.kde.plasma.core 2.1
import QtQuick.XmlListModel 2.5
import org.kde.plasma.networkmanagement 0.2

Item {
    id:getjsonDataMain  // scripts to get json data
    readonly property string symbols:"symbols=^DJI,^IXIC,^GSPC,CL=F,GC=F,^TNX"
    readonly property string url1:"https://query1.finance.yahoo.com/v8/finance/spark?"+symbols+"&range=1d&interval=15m&indicators=close&includeTimestamps=false&includePrePost=false&corsDomain=finance.yahoo.com&.tsrc=finance&format=json"
    readonly property var finSymbols:["^DJI","^IXIC","^GSPC","CL=F","GC=F","^TNX"] //6
    property var finClose:[]
    property var finPrevClose:[]
    property string mdate:"1234"
    property var curDate:Qt.formatDate(checkTimer.data["Local"]["DateTime"],"ddd")
    property var curTime:Qt.formatTime(checkTimer.data["Local"]["DateTime"],"hh:mm")
    readonly property var days:["Mon","Tue","Wed","Thu","Fri"]
    property var mstartTime:'08:35'
    property var mendTime:'15:20'
    property var gmailMsgs:{}
    readonly property string url2:"/tmp/gmail.json"
    property var rssFeed:[]
    property int articleCount:0

    readonly property string url3:"/tmp/gcal.json"
    property var calData:[]
    property var calDayEvents:[]
    property var calDayDates:[]

    readonly property bool offSeason:true
    readonly property string url4:offSeason ? "https://site.api.espn.com/apis/site/v2/sports/baseball/mlb/scoreboard" : "https://site.api.espn.com/apis/site/v2/sports/basketball/nba/scoreboard"
    //property string url2:"https://site.api.espn.com/apis/site/v2/sports/basketball/nba/scoreboard"
    readonly property string favTeam:offSeason ? "Houston Astros" : "Houston Rockets"
    //property string favTeam:"Houston Rockets"

    readonly property string url5: "/tmp/weather.json"
    property var forecastDays:["-","-","-","-","-"]
    property var forecastIcons:[]
    property var forecastRains:["-","-","-","-","-"]
    property var forecastTemps:["-","-","-","-","-"]
    property string forecastNarrative:""
    property string weatherIcon:""
    property string weatherTemp:"NA"
    property string weatherRealFeel:""
    property string weatherHumidity:""
    property string weatherWinds:"NA"
    property string lastUpdate:"na"
    property bool weatherWarnings:false
    property string weatherConditions:"--"
    property string weatherAlertText:""
    property string radarImage:"SETX1_1280.jpg"

    property int key:-1
    property int gameIntervalValue:20*60*1000
    property string gameType:"MLB"
    property string gameStatusState:""
    property string gameStatusDetail:""
    property string gameStatusDescription:""
    property string gamePeriod:""
    property string gameClock:""
    property string gameDate:""
    property string gameHeadline:""
    property string gameBoxScoresURL:""
    property string gamePlayoffRecord:""
    property string gameAltHeadline:""
    property bool homeTeamWinner:false
    //property string homeTeamName:""
    property string homeTeamAbrev:""
    property string homeTeamLogo:""
    property string homeTeamScore:""
    property string homeTeamRecord:""
    //property string awayTeamName:""
    property string awayTeamAbrev:""
    property string awayTeamLogo:""
    property string awayTeamScore:""
    property string awayTeamRecord:""
    property bool awayTeamWinner:false

    Component.onCompleted: {
        mdate=getDate();
        radarImage="https://cdns.abclocal.go.com/three/ktrk/weather/16_9/SETX1_1280.jpg"
        getData(url1);
        getData(url2);
        getData(url3);
        getData(url4);
        getData(url5);
    }

     XmlListModel {
        id: xmlModel
        source: "/tmp/nbc.xml"
        //source:"/tmp/nbc.rss"
        query: "/rss/channel/item"
        //namespaceDeclarations: "declare default element namespace 'http://www.w3.org/2005/Atom';"

        XmlRole { name: "title"; query: "title/string()";isKey: true }
        XmlRole { name: "link"; query: "link/string()" }
        onStatusChanged: {
            status==1 ?  rssFeed=xmlModel : ""
    }
   }

    function formatNumber(num) {
        return num.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1,')
    }

    function degToCompass(num) {
        let val = Math.floor((num / 22.5) + 0.5);
        let arr = ["N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW"];
        return arr[(val % 16)];
    }

    function getData(url){  // get json data sources
        let xhr = new XMLHttpRequest;
        xhr.open("GET",url); // set Method and File  true=asynchronous
        xhr.responseType = 'json';
        xhr.setRequestHeader('Content-Type', 'application/json');
        xhr.onreadystatechange = function () {
            if(xhr.readyState == 4){ // if request_status == DONE
                if (xhr.status == 200) {
                    if (url===url1) {
                        let market={};
                        market=JSON.parse(xhr.responseText);
                        processMarketData(market);
                        market=null;
                    }
                    else if (url===url2) {
                        gmailMsgs=JSON.parse(xhr.responseText);
                    }
                    else if (url===url3) {
                        calData = JSON.parse(xhr.responseText);
                        eventsForDate();
                    }
                    else if (url===url4) {
                        let scores={};
                        scores=JSON.parse(xhr.responseText);
                        processGameData(scores);
                        scores=null;
                    }
                    else if (url===url5) {
                        let weatherData={};
                        weatherData=JSON.parse(xhr.responseText);
                        processWeatherData(weatherData);
                        weatherData=null;
                    }
                }
            }
        }
        xhr.send();
        return null;
    }

    function eventsForDate() {
        calDayEvents=[];
        calDayDates=[];
        let events=[];
        let dates=[];
        for (let i = 0; i < calData.length; i++) {
         if (calData[i].start == Qt.formatDate(checkTimer.data["Local"]["DateTime"],"yyyy-MM-dd")) {
             events.push(calData[i].summary)
             dates.push(calData[i].start)
         }
        }
        calDayEvents=events
        calDayDates=dates
        events=null;
        dates=null;
        calData=null;
        return null;
    }

    function processGameData (scores) {
        if (typeof(scores) != "undefined") {
            if (Object.keys(scores.events).length > 0 ) { // check if any data exists
                for (let i = 0; i < Object.keys(scores.events).length; i++) {
                    if (scores.events[i].competitions[0].competitors[0].team.displayName===favTeam)  {
                        key=i;
                        break;
                    }
                    else if (scores.events[i].competitions[0].competitors[1].team.displayName===favTeam)  {
                        key=i;
                        break;
                    }
                    if (gameStatusState == "in") {
                        key=key }
                    else key=Math.floor(Math.random() * Object.keys(scores.events).length) // else no fav team game, pick random game.
                }
                gameType=scores.leagues[0].abbreviation !=undefined ? scores.leagues[0].abbreviation : "-"
                gameStatusState=scores.events[key].competitions[0].status.type.state !=undefined ? scores.events[key].competitions[0].status.type.state:"-"
                gameStatusDetail=scores.events[key].competitions[0].status.type.shortDetail !=undefined ? scores.events[key].competitions[0].status.type.shortDetail:"-"
                gameStatusDescription=scores.events[key].competitions[0].status.type.description !=undefined ? scores.events[key].competitions[0].status.type.description:"-"
                gamePeriod=scores.events[key].competitions[0].status.period !=undefined ? scores.events[key].competitions[0].status.period : "-"
                gameClock=scores.events[key].competitions[0].status.displayClock !=undefined ? scores.events[key].competitions[0].status.displayClock:"-"
                gameDate=scores.events[key].competitions[0].date !=undefined ? scores.events[key].competitions[0].date :"-"
                gamePlayoffRecord=scores.events[key].competitions[0].hasOwnProperty("series") ? scores.events[key].competitions[0].series.summary:""
                gameAltHeadline=(typeof scores.events[key].competitions[0].notes[0]!="undefined") ? scores.events[key].competitions[0].notes[0].headline : ""
                gameHeadline=scores.events[key].competitions[0].hasOwnProperty("headlines") ? scores.events[key].competitions[0].headlines[0].shortLinkText : (typeof scores.events[key].competitions[0].notes[0] == "undefined") ? "" : gameAltHeadline+"    "+gamePlayoffRecord
                //gameHeadline=scores.events[key].competitions[0].hasOwnProperty("headlines") ? scores.events[key].competitions[0].headlines[0].shortLinkText : ""
                gameBoxScoresURL=scores.events[key].links[0].href;

                //homeTeamName=scores.events[key].competitions[0].competitors[0].team.displayName;
                homeTeamAbrev=scores.events[key].competitions[0].competitors[0].team.abbreviation !=undefined ? scores.events[key].competitions[0].competitors[0].team.abbreviation : "-"
                homeTeamLogo=scores.events[key].competitions[0].competitors[0].team.logo !=undefined ? scores.events[key].competitions[0].competitors[0].team.logo : ""
                homeTeamScore=scores.events[key].competitions[0].competitors[0].score !=undefined ? scores.events[key].competitions[0].competitors[0].score : 0
                homeTeamRecord=scores.events[key].competitions[0].competitors[0].hasOwnProperty("records") ? scores.events[key].competitions[0].competitors[0].records[0].summary:""
                homeTeamWinner=scores.events[key].competitions[0].competitors[0].winner !=undefined ? scores.events[key].competitions[0].competitors[0].winner:false

                //awayTeamName=scores.events[key].competitions[0].competitors[1].team.displayName;
                awayTeamAbrev=scores.events[key].competitions[0].competitors[1].team.abbreviation !=undefined ? scores.events[key].competitions[0].competitors[1].team.abbreviation: ""
                awayTeamLogo=scores.events[key].competitions[0].competitors[1].team.logo !=undefined ? scores.events[key].competitions[0].competitors[1].team.logo : ""
                awayTeamScore=scores.events[key].competitions[0].competitors[1].score !=undefined ? scores.events[key].competitions[0].competitors[1].score : 0
                awayTeamRecord=scores.events[key].competitions[0].competitors[1].hasOwnProperty("records") ? scores.events[key].competitions[0].competitors[1].records[0].summary:""
                awayTeamWinner=scores.events[key].competitions[0].competitors[1].winner !=undefined ? scores.events[key].competitions[0].competitors[1].winner:false

                gameInterval ();
            }
        }
        scores=null;
        return null;
    }

    function processMarketData (mkt) {
        if (typeof(mkt) != "undefined") {
            let x1=[];
            let y1=[];
            x1[0]=mkt[finSymbols[0]].close.slice(-1)[0]!=null ? mkt[finSymbols[0]].close.slice(-1)[0].toFixed(2):mkt[finSymbols[0]].close.slice(-2)[0].toFixed(2);
            x1[1]=mkt[finSymbols[1]].close.slice(-1)[0]!=null ? mkt[finSymbols[1]].close.slice(-1)[0].toFixed(2):mkt[finSymbols[1]].close.slice(-2)[0].toFixed(2);
            x1[2]=mkt[finSymbols[2]].close.slice(-1)[0]!=null ? mkt[finSymbols[2]].close.slice(-1)[0].toFixed(2):mkt[finSymbols[2]].close.slice(-2)[0].toFixed(2);
            x1[3]=mkt[finSymbols[3]].close!=null ? mkt[finSymbols[3]].close.slice(-1)[0] !=null ? mkt[finSymbols[3]].close.slice(-1)[0].toFixed(2):mkt[finSymbols[3]].close.slice(-2)[0].toFixed(2) : mkt[finSymbols[3]].previousClose.toFixed(2);
            x1[4]=mkt[finSymbols[4]].close!=null ? mkt[finSymbols[4]].close.slice(-1)[0] !=null ? mkt[finSymbols[4]].close.slice(-1)[0].toFixed(2):mkt[finSymbols[4]].close.slice(-2)[0].toFixed(2) : mkt[finSymbols[4]].previousClose.toFixed(2);
            x1[5]=mkt[finSymbols[5]].close!=null ? mkt[finSymbols[5]].close.slice(-1)[0] !=null ? mkt[finSymbols[5]].close.slice(-1)[0].toFixed(2):mkt[finSymbols[5]].close.slice(-2)[0].toFixed(2) : mkt[finSymbols[5]].previousClose.toFixed(2);

            finClose=x1;

            y1[0]=mkt[finSymbols[0]].previousClose.toFixed(2);
            y1[1]=mkt[finSymbols[1]].previousClose.toFixed(2);
            y1[2]=mkt[finSymbols[2]].previousClose.toFixed(2);
            y1[3]=mkt[finSymbols[3]].previousClose.toFixed(2);
            y1[4]=mkt[finSymbols[4]].previousClose.toFixed(2);
            y1[5]=mkt[finSymbols[5]].previousClose.toFixed(2);

            finPrevClose=y1;
        }
        x1=null;
        y1=null;
        mkt=null;
        return null;
    }

    function processWeatherData(weatherData) {
        if (typeof(weatherData.current.validTimeUtc) != "undefined") {
            // current conditions data
            let d1=[];
            let i1=[];
            let r1=[];
            let t1=[];

            lastUpdate=weatherData.current.validTimeUtc != "undefined" ? Qt.formatTime(new Date(weatherData.current.validTimeUtc*1000),"h:mm ap"):"-"
            weatherIcon=weatherData.current.iconCode != "undefined" ? "../icons/"+weatherData.current.iconCode+".png" : "../icons/na.png"
            weatherTemp=weatherData.current.temperature != "undefined" ? weatherData.current.temperature+"°" : "-"
            weatherConditions=weatherData.current.wxPhraseMedium
            weatherRealFeel="Real Feel:  "+weatherData.current.temperatureFeelsLike+"°"
            weatherHumidity="     R/H:  "+weatherData.current.relativeHumidity+"%"
            weatherWinds="      Wind:  "+weatherData.current.windDirectionCardinal+"  at "+weatherData.current.windSpeed

            if (typeof(weatherData.forecast.calendarDayTemperatureMax[0]) != "undefined") {
            for (var i = 0; i < Object.keys(weatherData.forecast.calendarDayTemperatureMax).length; i++) {
                d1[i]=Qt.formatDate(new Date(weatherData.forecast.validTimeUtc[i]*1000),"ddd")
                i1[i]="../icons/"+weatherData.forecast.daypart[0].iconCode[i < 1 ? i+1 : i*2]+".png";
                r1[i]=Math.floor((weatherData.forecast.daypart[0].precipChance[i < 1 ? i+1 : i*2])/10)*10+"%"
                t1[i]=weatherData.forecast.calendarDayTemperatureMin[i]+"°"+" | "+weatherData.forecast.calendarDayTemperatureMax[i]+"°"
                //t1[i]=forecastData.daypart[0].temperature[i%2 ? i:i+1] ? forecastData.daypart[0].temperature[i%2 ? i:i+1]+"°"+" | "  : forecastData.calendarDayTemperatureMin[i]+"°"+" | "+ forecastData.daypart[0].temperature[i%2 ? i+1:i] ? forecastData.daypart[0].temperature[i%2 ? i+1:i]+"°" : forecastData.calendarDayTemperatureMax[i]+"°"
                //t1[i]=forecastData.daypart[0].temperature[i] ? forecastData.daypart[0].temperature[i]+"°" : forecastData.calendarDayTemperatureMin[i]+"°" +" | "+ forecastData.daypart[0].temperature[i+1] ? forecastData.daypart[0].temperature[i+1]+"°" : forecastData.calendarDayTemperatureMax[i]+"°"
            }
            forecastNarrative=weatherData.forecast.narrative[0]
            forecastDays=d1;
            forecastIcons=i1;
            forecastRains=r1;
            forecastTemps=t1;
            d1=null;
            i1=null;
            r1=null;
            t1=null;
        }
        if (typeof(weatherData.warnings) != "undefined") {
            weatherWarnings=weatherData.warnings.hasOwnProperty("alerts"); //? true:false // check if alert exists
            weatherAlertText=weatherWarnings ? "⚠ "+weatherData.warnings.alerts[0].eventDescription : ""
            }
        }
        return null;
    }


    function processNewsData(newsStories) {
        let x1=[];
        let y1=[];
        for (var i = 0; i < Object.keys(newsStories.result).length; i++) {
            //x1[i]=newsStories.result[i].headline.replace("undefined","").replace(/\d{1,2}[\-|\.|\/]\d{1,2}[\-|\.|\/]\d{2,4}/g,"")
            x1[i]=newsStories.result[i].headline.replace(Qt.formatDate(new Date(new Date().setDate(new Date().getDate()-1)),"MMMM d, yyyy"),"").replace(Qt.formatDate(new Date(),"MMMM d, yyyy")).replace("undefined","")
            y1[i]=newsStories.result[i].url
        }
        newsArticles=x1;
        newsLinks=y1
        x1=null;
        y1=null;
        newsStories=null;
        return null;
    }


    function getOrdinal(n) {            // assigns superfix to inning
        let s=["th","st","nd","rd"],
        v=n%100;
        return (s[(v-20)%10]||s[v]||s[0]);
    }

    function gameState() {
        if (gameStatusState == "pre") {
            return (Qt.formatDateTime(new Date(gameDate),"h:mm ap"));
        }
        else if (gameStatusState == "in") {
            if (gameStatusDescription!="In Progress") {
                return (gameStatusDetail);
            }
            return (gamePeriod+getOrdinal(gamePeriod));
        }
        else {
            return (gameStatusDetail);
        }
    }

    function getDate () {
        let today=new Date().toLocaleString("dd/mm/yy");
        return today
    }

    function checkDate() {  // check if market is open hours/days
        //curDate=Qt.formatDate(new Date(),"ddd");
        //curTime=Qt.formatTime(new Date(),"hh:mm");
        return (curTime > mstartTime && curTime < mendTime) && days.includes(curDate)
    }

    function gameInterval () {
        let currentHour=new Date(checkTimer.data["Local"]["DateTime"]).getHours();
        //var currentMins=new Date(checkTimer.data["Local"]["DateTime"]).getMinutes();
        let gameHour=new Date(gameDate).getHours();
        //var gameMins=new Date(gameDate).getMinutes();

        if (gameStatusState === "in") {
            if (gameStatusDetail==="Delayed") { // game delayed check again in 1:15 //
                gameIntervalValue= 1*75*60*1000;
                return null;
            }
            gameIntervalValue = 10*60*1000; //active game 10 min updates;
            return null;
        }

        if (gameStatusState === "pre") {
            //if ((Qt.formatDateTime(checkTimer.data["Local"]["DateTime"],"hhmm")-Qt.formatDateTime(new Date(gameDate),"hhmm")) < -15) {
            if (Math.abs(currentHour-gameHour) < 2) {
                gameIntervalValue = 20*60*1000;
                return null;
            }
            gameIntervalValue = 40*60*1000;
        }
        if (gameStatusState === "post") {
            gameIntervalValue =  2*60*60*1000;
            return null;
        }

        //else gameIntervalValue = 60*60*1000;
        return null;
    }

    function refreshData () {
        mdate=getDate();
        radarImage="SETX1_1280.jpg";
        radarImage="https://cdns.abclocal.go.com/three/ktrk/weather/16_9/SETX1_1280.jpg";
        getData(url1);
        getData(url2);
        getData(url3);
        getData(url4);
        getData(url5);
        xmlModel.reload();
        articleCount=0
        dataTimer.restart();
        gameTimer.restart();
        return null;
    }


    property var dataTimer:Timer {                  // timer to trigger update refresh data
        interval: 15 * 60 * 1000 // every 15 minutes
        running: true
        repeat:  true
        triggeredOnStart:false
        onTriggered: {
            radarImage="SETX1_1280.jpg"
            radarImage="https://cdns.abclocal.go.com/three/ktrk/weather/16_9/SETX1_1280.jpg"
            getData(url2);
            getData(url3);
            getData(url5);
            xmlModel.reload();
            if (checkDate()) {
                getData(url1);
                mdate=getDate();
            }
            if ( articleCount > xmlModel.count-10) {
                articleCount=0 }
            else articleCount+=1
        }
    }

    property var gameTimer: Timer {
        interval: gameIntervalValue
        running:true
        repeat:true
        triggeredOnStart:false
        onTriggered:getData(url4);
    }

    property var suspendTimer:Timer{        // timer to trigger update after wake from suspend mode
        interval: 30*1000                  // delay 20 secs for suspend to resume
        running: false
        repeat:  false
        triggeredOnStart:false
        onTriggered:refreshData();
    }

    NetworkStatus {   // check if network changed for resume from sleep mode...
        id: networkStatus       // systemctl suspend to test
        onNetworkStatusChanged: suspendTimer.start();
    }

    DataSource {
        id: checkTimer
        engine: "time"
        connectedSources: ["Local"]
        interval: 1000
    }
}
