import QtQuick 2.9
import org.kde.plasma.core 2.1

Item {
    id:gmailData
    height:ev.visible ? 100 : 80
    width:340
    property int today:Qt.formatDate(timeSource.data["Local"]["DateTime"],"MMdd")

    Item {
        id:dateTime
        anchors.top:gmailData.top
        anchors.horizontalCenter:gmailData.horizontalCenter
        height:ev.visible ? 60 : 35  // hide events if blank

        Text {
            text: Qt.formatTime(timeSource.data["Local"]["DateTime"],"h:mm ap")
            color:Theme.textColor
            font.pointSize:11
            anchors.horizontalCenter:parent.horizontalCenter

            Text {
                text:Qt.formatDate(timeSource.data["Local"]["DateTime"],"dddd - MMMM  d")
                color:Theme.textColor
                font.pointSize:11
                anchors.horizontalCenter:parent.horizontalCenter
                anchors.top:parent.bottom

                MouseArea {
                    anchors.fill: parent
                    hoverEnabled: true
                    onEntered: parent.color=Theme.viewHoverColor
                    onExited: parent.color=Theme.textColor
                    onClicked:  Qt.openUrlExternally("https://calendar.google.com/calendar/u/0/r")
                    cursorShape :Qt.PointingHandCursor
                }
            }

            Text {
                id:ev
                text:jsonData.calDayEvents[0] ? jsonData.calDayEvents[0]:""
                anchors.horizontalCenter:parent.horizontalCenter
                anchors.top:parent.bottom
                anchors.topMargin:25
                color:Theme.textColor
                antialiasing : true
                font.pointSize: 11
                visible:jsonData.calDayEvents[0] === undefined ? false : true
            }
        }
    }

    Rectangle {
        id:ts7
        anchors.horizontalCenter:parent.horizontalCenter
        anchors.top:dateTime.bottom
        anchors.topMargin:10
        width: parent.width*.95
        height: .5
        color: "#5b565d"
        antialiasing : true
    }

    Row {
        id:gmail
        spacing:15
        width:330
        anchors.top:ts7.bottom
        anchors.topMargin:10
        anchors.left:parent.left

        Item {
            id:gmailItem
            anchors.leftMargin:70
            width:84
            height:36
            anchors.left:parent.left
            anchors.top:parent.top

            Image {
                id:gmailIcon
                source:"gmail.png"
                width:20
                height:20
                smooth:true
            }

            Text {
                id:gmailTitle
                text:jsonData.formatNumber(jsonData.gmailMsgs.gmail)
                font.pointSize:10
                antialiasing : true
                anchors.left:gmailIcon.right
                anchors.top:gmailIcon.top
                anchors.leftMargin:10
                anchors.topMargin:1
                color: Theme.textColor
            }

            MouseArea {
                anchors.fill: gmailItem
                hoverEnabled: true
                onEntered: gmailTitle.color=Theme.viewHoverColor
                onExited: gmailTitle.color=Theme.textColor
                onClicked:  Qt.openUrlExternally("https://mail.google.com/mail/u/0/#inbox")
                cursorShape :Qt.PointingHandCursor
            }
        }

        Item {
            id:agendaItem
            anchors.left:gmailItem.right
            anchors.top:gmailItem.top
            width:84
            height:36

            Image {
                id:calIcon
                source:"google-agenda.png"
                width:20
                height:20
                smooth:true
            }

            Text {
                id:calTitle
                text:jsonData.formatNumber(jsonData.gmailMsgs.gcal)
                font.pointSize:10
                antialiasing : true
                anchors.left:calIcon.right
                anchors.top:calIcon.top
                anchors.leftMargin:10
                anchors.topMargin:1
                color: Theme.textColor
            }
            MouseArea {
                anchors.fill: agendaItem
                hoverEnabled: true
                onEntered: calTitle.color=Theme.viewHoverColor
                onExited: calTitle.color=Theme.textColor
                onClicked:  Qt.openUrlExternally("https://calendar.google.com/calendar/u/0/r/agenda")
                cursorShape :Qt.PointingHandCursor
            }
        }

        Item {
            id:voiceItem
            anchors.left:agendaItem.right
            anchors.top:agendaItem.top
            width:84
            height:36

            Image {
                id:voiceIcon
                source:"google-voice.png"
                width:20
                height:20
                smooth:true
            }

            Text {
                id:voiceTitle
                text:jsonData.formatNumber(jsonData.gmailMsgs.gvoice)
                font.pointSize:10
                antialiasing : true
                anchors.left:voiceIcon.right
                anchors.top:voiceIcon.top
                anchors.leftMargin:10
                anchors.topMargin:1
                color: Theme.textColor
            }
            MouseArea {
                anchors.fill: voiceItem
                hoverEnabled: true
                onEntered: voiceTitle.color=Theme.viewHoverColor
                onExited: voiceTitle.color=Theme.textColor
                onClicked:  Qt.openUrlExternally("https://voice.google.com/u/0/messages")
                cursorShape :Qt.PointingHandCursor
            }
        }
    }

    DataSource {
        id: timeSource
        engine: "time"
        connectedSources: ["Local"]
        interval: 1100
    }
}
