import QtQuick 2.9
import QtQuick.Layouts 1.5
import org.kde.plasma.core 2.1
import QtGraphicalEffects 1.5

Item {
    id: weatherData
    width: 340

    //GetData{id:jsonData}

    function updateWeather () {
        jsonData.radarImage="SETX1_1280.jpg";
        jsonData.radarImage="https://cdns.abclocal.go.com/three/ktrk/weather/16_9/SETX1_1280.jpg";
        jsonData.getData(jsonData.url5);
    }

    Text {
        id:updateTime
        text:jsonData.lastUpdate
        color:"gray"
        antialiasing : true
        font.pointSize:8
        anchors.top:weatherData.top
        anchors.right:weatherData.right
        anchors.rightMargin:5


        MouseArea {
            id: updatemouseArea
            anchors.fill: parent
            cursorShape: Qt.PointingHandCursor
            hoverEnabled:true
            onEntered:updateTime.color=Theme.viewHoverColor
            onExited:updateTime.color="gray"
            onClicked: updateWeather ();
        }
    }
    Row {
        id:conditions
        anchors.top:weatherData.top
        anchors.left:weatherData.left
        anchors.leftMargin:7
        spacing:5
        height:40

        Image {
            id:iconCode
            source:jsonData.weatherIcon
            width:42
            height:42
            smooth:true
            anchors.top:conditions.top
        }

        Text {
            id:temp
            anchors.top:iconCode.top
            anchors.topMargin:5
            text:"  "+jsonData.weatherTemp
            color:Theme.viewTextColor
            font.pointSize:18
            antialiasing : true
        }

        Text {
            id:summary
            anchors.bottom:temp.bottom
            text:"  "+jsonData.weatherConditions
            font.capitalization: Font.Capitalize
            // if jsonData.weather alert show that instead of summary
            color:Theme.viewTextColor
            font.pointSize:14
            antialiasing : true
        }
    }

    Column {
        id:currents
        anchors.top:conditions.bottom
        anchors.left:weatherData.left
        anchors.leftMargin:10
        spacing:5
        height:50

        Text {
            id:story
            topPadding:8
            text:jsonData.weatherWarnings ? jsonData.weatherAlertText : jsonData.forecastNarrative
            elide: Text.ElideRight
            wrapMode: Text.NoWrap
            Layout.fillWidth : true
            width:315
            color:Theme.viewTextColor
            font.pointSize:11
            antialiasing : true

            MouseArea {
                id: mouseArea
                anchors.fill: parent
                cursorShape: jsonData.weatherWarnings ? Qt.PointingHandCursor : Qt.ArrowCursor
                hoverEnabled: jsonData.weatherWarnings ? true : false
                onEntered:jsonData.weatherWarnings ? story.color=Theme.viewHoverColor: story.color=Theme.viewTextColor
                onExited:jsonData.weatherWarnings ? story.color=Theme.viewTextColor : story.color=Theme.viewTextColor
                onClicked: {
                    Qt.openUrlExternally("https://alerts.weather.gov/cap/wwaatmget.php?x=TXC201&y=1")
                }
            }
        }

        Text {
            id:currentReadings
            text:jsonData.weatherRealFeel+jsonData.weatherHumidity+jsonData.weatherWinds
            wrapMode:Text.WordWrap
            Layout.fillWidth : true
            topPadding:6
            width:315
            color:Theme.viewTextColor
            font.pointSize:11
            antialiasing : true
        }
    }

    Rectangle {
        id:ts
        anchors.top:currents.bottom
        anchors.topMargin:12
        anchors.horizontalCenter:weatherData.horizontalCenter
        width: weatherData.width*.95
        height:.5
        color: "#5b565d"
        antialiasing : true
    }

    Row {
        id:forecast
        anchors.top:ts.bottom
        anchors.horizontalCenter:ts.horizontalCenter
        topPadding:10
        spacing:44
        Repeater {
            id:r1
            model: 5
            Text {
                text:(typeof(jsonData.forecastDays[index]) != "undefined") ? jsonData.forecastDays[index] : "-"
                color:Theme.viewTextColor
                font.pointSize:11
                font.bold:true
                antialiasing : true

                Image {
                    source:(typeof(jsonData.forecastIcons[index]) != "undefined") ? jsonData.forecastIcons[index] : "../icons/na.png"
                    width:36
                    height:36
                    smooth:true
                    anchors.top:parent.bottom
                    anchors.horizontalCenter:parent.horizontalCenter

                    Text {
                        text:(typeof(jsonData.forecastRains[index]) != "undefined") ? jsonData.forecastRains[index] : "-"
                        color:Theme.viewTextColor
                        anchors.horizontalCenter:parent.horizontalCenter
                        anchors.top:parent.bottom
                        anchors.topMargin:5
                        font.pointSize:10
                        antialiasing : true

                        Text {
                            text:(typeof(jsonData.forecastTemps[index]) != "undefined") ? jsonData.forecastTemps[index] : "-"
                            color:Theme.viewTextColor
                            anchors.horizontalCenter:parent.horizontalCenter
                            anchors.top:parent.bottom
                            anchors.topMargin:5
                            font.pointSize:10
                            antialiasing : true
                        }
                    }
                }
            }
        }
    }

   Rectangle {
        id:ts1
        anchors.top:forecast.bottom
        anchors.horizontalCenter:weatherData.horizontalCenter
        anchors.topMargin:85
        width: weatherData.width*.95
        height: .5
        color: "#5b565d"
        antialiasing : true
    }

    Column {
        id:radar_info
        anchors.top:ts1.bottom
        anchors.topMargin:10
        anchors.left:weatherData.left
        spacing:2

        Image {
            id:radar
            source:jsonData.radarImage
            smooth: false // allow opacity mask == false
            width: weatherData.width
            fillMode:Image.PreserveAspectFit
            visible: false
            cache:true
        }

        OpacityMask {
            anchors.fill: radar
            source: radar
            maskSource: Rectangle {    // whats behind mask is visible, adds rounded corners to radar image
                width: radar.width
                height: radar.height
                radius: 8
                antialiasing : true
                visible: false // this also needs to be invisible or it will cover up the image
            }

            MouseArea {
                id:ma1
                anchors.fill: parent
                acceptedButtons: Qt.LeftButton | Qt.MiddleButton
                hoverEnabled: true
                cursorShape:Qt.PointingHandCursor
                onClicked: {
                    (mouse.button==Qt.LeftButton) ? Qt.openUrlExternally("https://abc13.com/weather/") : updateWeatherRadar ()
                }
            }
        }
    }

    function updateWeatherRadar () {
        jsonData.radarImage="SETX1_1280.jpg"
        jsonData.radarImage="https://cdns.abclocal.go.com/three/ktrk/weather/16_9/SETX1_1280.jpg"
    return null;
    }
}
